import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.*;

// Establish Multi-Threaded Connection to intraServer
class IntranetConnection extends ClientConnection {

	public IntranetConnection (Socket client_socket, ThreadGroup threadgroup,
			int priority, Vulture vulture)
	{
		super (client_socket, threadgroup, priority, vulture);
	}


	public void run () {
		int failed_connects = 0;
		boolean connected = false;
		try {
			unicodein = new InputStreamReader (client.getInputStream ());
			unicodeout = new OutputStreamWriter (client.getOutputStream ());
			in = new BufferedReader (unicodein);
			out = new PrintWriter (unicodeout);
			IntranetConnectionInterface intracon = new OracleConnectionHandler ();
			Hashtable procedures = new Hashtable ();
         for (;;) {
				synchronized (in) {
					String arguments [] = new String [MAX_ARGS];
					String line = in.readLine ();
					if (line == null) break;
					for (int i = 1; i<5; i++) 
						arguments[i] = new String();
					StringTokenizer st = new StringTokenizer (line);
					for (int argc = 1; st.hasMoreTokens () && argc < MAX_ARGS; 
							argc++)
						arguments [argc] = new String (st.nextToken ());
					String COMMAND = arguments [1].toUpperCase ();
					if (COMMAND.equals (CONNECT_STRING)) {
						USR = arguments [2].toUpperCase ();
						PASSWD = arguments [3].toUpperCase ();
						APPLICATION = arguments [4].toUpperCase ();
						connected = false;
						try {
							// if already connected, kill old connection
							if (con != null) con.close ();
							// open a database connection
							con = intracon.establishConnection (USR,PASSWD,APPLICATION);
							procedures = intracon.valid_procedures (con,APPLICATION);
						}
						catch (SQLException e) {  
							if (++failed_connects == MAX_ATTEMPTS) {
								out.println ("Denied access");
								break;
							}
							else {
								out.println ("Failed to connect to Intranet Server");
								out.flush ();
								continue;
							}
						}
						out.println (USR + "/" + client.getInetAddress ().getHostName ()+
										" logged in to " + APPLICATION);
						connected = true;
					}
					else if (COMMAND.equals (QUIT_STRING)) {
						out.println ("Connection to Phone Directory closed.");
						break;
					}
					else if (!connected)
						out.println ("ERROR: Invalid Command...");
					else {

						// connection has timed out - reopen with old details
						if (con == null) {
							con = intracon.establishConnection (USR,PASSWD,APPLICATION);
							synchronized (idle_time) {
								idle_time = new Integer(0);
							}
						}

						if (!valid_command (COMMAND,procedures)) {
							out.println ("ERROR: invalid procedure name");
							out.flush ();
							continue;
						}

						String args[];
						if (line.length () == COMMAND.length ())
							args = new String [MAX_VALS];
						else
							args = get_args (COMMAND,line);
						
						boolean args_ok = check_args ((Vector)procedures.get(COMMAND),args);
						if (!args_ok) {
							out.flush ();
							continue;
						}
						try {
							if (COMMAND.endsWith("_QUERY")) { // it is a query
								Vector results = intracon.select 
									(con, (Vector)procedures.get (COMMAND), COMMAND, args);
								Enumeration vr = results.elements ();
								int cnt;
								for (cnt = 1; vr.hasMoreElements (); cnt++) {
									String xxx = (String)vr.nextElement ();
									out.println(xxx);
								}
								if (cnt == 1)
									out.println("WARNING: no records matched selection");
							} else if 
								(COMMAND.endsWith ("_UPDATE") || COMMAND.endsWith ("_ADD")
									|| COMMAND.endsWith("_DELETE"))
									intracon.update (con, (Vector)procedures.get (COMMAND), 
									COMMAND, args);
							else
								out.println("ERROR: unsupported procedure");
						}
						catch (SQLException e) {
						//	out.println("ERROR: Command failed to execute");
							out.println(e.getMessage());
							out.flush();
							continue;
						}
						catch (Exception e) {
							out.println(e.getMessage());
						}
						out.println("OK");
					} // synchronized
				} 
				out.flush ();
			}
			out.flush ();
		} catch (Exception e) {}
		finally {
			close_connection();
		}
	}


	private void close_connection() {
		try {
			if (con != null) con.close();
				out.close ();
				in.close ();
				unicodeout.close ();
				unicodein.close ();
			}
			catch (Exception e) {}
			synchronized (vulture) { 
				vulture.notify(); 
			}
		}
}

