import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.*;
import java.math.*;
import procArgs;

// Establish Multi-Threaded Connection to intraServer
class MT_Connection extends Thread {
   static final int MAX_ARGS = 5;
   static final int MAX_VALS = 100;
   static final int MAX_ATTEMPTS = 3;
   static final String QUIT_STRING = "QUIT";
   static final String CONNECT_STRING = "CONNECT";
   static int connection_number = 0;
   protected Socket client;
   protected Vulture vulture;
   protected InputStreamReader unicodein;
   protected BufferedReader in;
   protected OutputStreamWriter unicodeout;
   protected PrintWriter out;
   protected Connection con;
   protected Statement stmt;
   protected CallableStatement cstmt;
   protected String USR = new String ();
   protected String PASSWD = new String ();
   protected String APPLICATION = new String ();

   public MT_Connection (Socket client_socket, ThreadGroup threadgroup,
                  int priority, Vulture vulture)
   {
      super (threadgroup, "Connection-" + connection_number++);
      this.setPriority (priority);
      client = client_socket;
      this.vulture = vulture;
      this.start ();
   }

   public void run () {
		int failed_connects = 0;
		boolean connected = false;

      try {
         unicodein = new InputStreamReader (client.getInputStream ());
         unicodeout = new OutputStreamWriter (client.getOutputStream ());
         in = new BufferedReader (unicodein);
         out = new PrintWriter (unicodeout);
         IntranetConnectionInterface intracon = new OracleConnectionHandler ();
			Hashtable procedures = new Hashtable ();

         for (;;) {
            String arguments [] = new String [MAX_ARGS];
            String line = in.readLine ().toUpperCase ();
            if (line == null) break;
            StringTokenizer st = new StringTokenizer (line);
            for (int argc = 1; st.hasMoreTokens () && argc < MAX_ARGS; argc++)
                arguments [argc] = new String (st.nextToken ());
            String COMMAND = arguments [1];

            if (COMMAND.equals (CONNECT_STRING)) {
                USR = arguments [2];
                PASSWD = arguments [3];
                APPLICATION = arguments [4];
                connected = false;
                try {
                    // if already connected, kill old connection
                    if (con != null) con.close ();
                    // open a database connection
                    con = intracon.establishConnection (USR,PASSWD,APPLICATION);
                    procedures = intracon.valid_procedures (out,con,APPLICATION);
                }
					catch (SQLException e) {  
						if (++failed_connects == MAX_ATTEMPTS) {
							out.println ("Denied access");
							break;
						}
						else {
							out.println ("Failed to connect to Intranet Server");
							out.flush ();
							continue;
						}
					}
               out.println (USR + "/" + client.getInetAddress ().getHostName ()+
									" logged in to " + APPLICATION);
               connected = true;
            }
            else if (COMMAND.equals (QUIT_STRING)) {
               out.println ("Connection to Phone Directory closed.");
               break;
            }
				else if (!connected) {
               out.println ("ERROR: Invalid Command...");
				}
				else {
					if (!valid_command (COMMAND,procedures)) {
						out.println ("ERROR: invalid procedure name");
						out.flush ();
						continue;
					}
					if (line.length () == COMMAND.length ()) {
						out.println ("ERROR: No arguments given with command");
						out.flush ();
						continue;
					}
					String args[] = get_args (COMMAND,line);
					boolean args_ok = check_args ((Vector)procedures.get(COMMAND),args);
					if (!args_ok) {
						out.flush ();
						continue;
					}
					try {
						Vector results = intracon.execute 
							(con, (Vector)procedures.get (COMMAND), COMMAND, args);
						Enumeration vr = results.elements ();
						while (vr.hasMoreElements ()) {
							String xxx = (String)vr.nextElement ();
							out.println(xxx);
						}
					}
					catch (SQLException e) {
						out.println("ERROR: Command failed to execute");
					}
					catch (Exception e) {
						out.println(e.getMessage());
					}
				} 
            out.flush ();
         }
         out.flush ();
      } catch (Exception e) {}
      finally {
         try {
            if (con != null) con.close();
            out.close ();
            in.close ();
            unicodeout.close ();
            unicodein.close ();
         }
         catch (SQLException e) {
				out.println("ERROR: unable to close connection");
			}
         catch (IOException e) {}
         synchronized (vulture) { 
				vulture.notify(); 
			}
      }
   }


   public String toString () {
      return this.getName () + " connected to: "
                  + client.getInetAddress ().getHostName ()
                  + ":" + client.getPort ();
   }


	private boolean valid_command (String command, Hashtable proc_names) {
		boolean valid = false;
		Enumeration proc_keys = proc_names.keys (); 
		while (proc_keys.hasMoreElements ()) {
			if (command.equals ((String)proc_keys.nextElement ()))
				valid = true;
		}
		if (!valid) return false;
		else return true;
	}
		

	private String[] get_args(String command, String command_line) {
		String values[] = new String[MAX_VALS];
		String value_line = command_line.substring(command.length () + 1);
		StringTokenizer pipes = new StringTokenizer (value_line,"|");
		for (int num = 1; pipes.hasMoreTokens () && num < MAX_VALS; num++)
			values[num] = new String (pipes.nextToken ());
		return values;
	}


	private boolean check_args (Vector proc_args, String[] values) {
		Enumeration arg_keys = proc_args.elements ();
		int arg_no = 1;
		boolean enough_args = true;
		boolean correct_type = true;
		while (arg_keys.hasMoreElements () && enough_args && correct_type)
		{
			procArgs testArg = (procArgs)arg_keys.nextElement ();
			if (values[arg_no] == null)
				enough_args = false;
			if (testArg.getType().equals ("NUMBER")) {
				try {
					int testnum = Integer.parseInt (values[arg_no]);
				}
				catch (NumberFormatException e) {
					correct_type = false;
				}
			}
			arg_no++;
		}
		if (values[arg_no] != null) {
			out.println ("ERROR: too many arguments provided");
			return false;
		}
		if (!enough_args) {
			out.println ("ERROR: insufficient arguments provided");
			return false;
		} 
		else if (!correct_type) {
			out.println ("ERROR: a number argument was given as a string");
			return false;
		} 
		return true;
	}

}

