import java.net.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import oracle.jdbc.driver.*;
import procArgs;

class OracleConnectionHandler implements IntranetConnectionInterface {
  
  static final String DRIVER = "oracle.jdbc.driver.OracleDriver";
  static final String URL = "jdbc:oracle:oci7:@Intranet";

  static {
    try {
      // try to load the JDBC driver
      Class.forName (DRIVER);
    } catch (ClassNotFoundException ex) {
      ex.printStackTrace ();
    }
  }

  public Connection establishConnection (String LOGIN, String PASSWD, String APPLICATION)
         throws SQLException {

    Connection con = DriverManager.getConnection (URL, LOGIN, PASSWD);
    CallableStatement cstmt = con.prepareCall("{call chk_perm(?, ?)}");
    cstmt.setString(1,LOGIN);
    cstmt.setString(2,APPLICATION);
    cstmt.executeUpdate();
    cstmt.close();
    return (con);
  }




	public Hashtable valid_procedures (Connection con, String app)  
		throws SQLException
	{
		Hashtable procedures = new Hashtable();
		CallableStatement call = 
			con.prepareCall ("{ ? = call intranet.get_procs (?) }");
		call.setString(2, app.toUpperCase());
		call.registerOutParameter(1, OracleTypes.CURSOR);
		call.execute ();
		ResultSet rset = (ResultSet)call.getObject (1);
	
		while (rset.next()) {
			String proc = rset.getString("OBJECT_NAME");
			if (!procedures.containsKey(proc))
				procedures.put(proc,new Vector());
		}

		call = con.prepareCall ("{ ? = call intranet.get_proc_args (?) }");
		call.setString(2, app.toUpperCase());
		call.registerOutParameter(1, OracleTypes.CURSOR);
		call.execute ();
		rset = (ResultSet)call.getObject (1);

		while (rset.next ()) {
			String proc = rset.getString("OBJECT_NAME");
			procArgs pa = new procArgs(
				rset.getString("ARGUMENT_NAME"),
				rset.getString("IN_OUT"),
				rset.getInt("SEQUENCE"),
				rset.getString("DATA_TYPE"));
			Vector v = (Vector)procedures.get(proc);
			v.addElement((Object)pa); 
		} 
		rset.close();
		call.close();
		return procedures;
  }


	public Vector select (Connection con, Vector procedure, String command,
									String []args)
		throws SQLException 
	{

		Vector returnValues = new Vector();
		String sql = "{ ? = call intranet." + command;
		Enumeration proc_args = procedure.elements();
		int arg_no;
		for (arg_no = 1; proc_args.hasMoreElements (); arg_no++) {
			if (arg_no == 1) sql += " (";
			if (arg_no > 1) sql += ", ";
			sql += "?";
			// to increment the proc_args.hasMoreElements() counter
			procArgs testArg = (procArgs)proc_args.nextElement ();
		}
		if (arg_no > 1) sql += ")";
		sql += " }";

		CallableStatement call = con.prepareCall (sql);
		

		// which argument we are setting in the callable statement
		int next_arg = 1; 

		proc_args = procedure.elements();
		for (arg_no = 2; proc_args.hasMoreElements (); arg_no++) {
			procArgs testArg = (procArgs)proc_args.nextElement ();
			if (testArg.getInOut ().equals("IN")) {
				if (testArg.getType ().equals ("NUMBER"))
					try {	
						int number_arg = Integer.parseInt (args [next_arg++]);
						call.setInt(arg_no, number_arg);
					}
					catch (NumberFormatException e) {}
				if (testArg.getType ().equals ("VARCHAR2"))
					call.setString (arg_no, args [next_arg++]);
			}
		}
		call.registerOutParameter(1, OracleTypes.CURSOR);
		call.execute();
		ResultSet rset = (ResultSet)call.getObject (1);
		ResultSetMetaData meta = rset.getMetaData();
	
		int row_count = 1;
		while (rset.next()) {
			String row_data = "";
			for (int col_count = 1; col_count <= meta.getColumnCount();
					col_count++)
				row_data += rset.getString(col_count) + "|";
			returnValues.addElement(row_data);	
			row_count++;
		}
		rset.close();
		call.close();

		return(returnValues);
	}


	public void update (Connection con, Vector procedure, String command,
									String []args)
		throws SQLException 
	{

		String sql = "{ call intranet." + command;
		Enumeration proc_args = procedure.elements();
		int arg_no;
		for (arg_no = 1; proc_args.hasMoreElements (); arg_no++) {
			if (arg_no == 1) sql += " (";
			if (arg_no > 1) sql += ", ";
			sql += "?";
			// to increment the proc_args.hasMoreElements() counter
			procArgs testArg = (procArgs)proc_args.nextElement ();
		}
		if (arg_no > 1) sql += ")";
		sql += " }";

		CallableStatement call = con.prepareCall (sql);
		

		// which argument we are setting in the callable statement
		int next_arg = 1; 

		proc_args = procedure.elements();
		for (arg_no = 1; proc_args.hasMoreElements (); arg_no++) {
			procArgs testArg = (procArgs)proc_args.nextElement ();
			if (testArg.getInOut ().equals("IN")) {
				if (testArg.getType ().equals ("NUMBER"))
					try {	
						int number_arg = Integer.parseInt (args [next_arg++]);
						call.setInt(arg_no, number_arg);
					}
					catch (NumberFormatException e) {}
				if (testArg.getType ().equals ("VARCHAR2"))
					call.setString (arg_no, args [next_arg++]);
			}
		}
		call.executeUpdate();
	
		call.close();
	}
}
